<?php

namespace Drupal\commerce_forteecom\PluginForm;

use Drupal;
use Drupal\commerce_forteecom\ForteEcomHelper;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Form\FormStateInterface;

class ForteEcomCheckoutForm extends BasePaymentOffsiteForm
{

  /**
   * @inheritdoc
   */
  public static function processRedirectForm(array $element, FormStateInterface $form_state, array &$complete_form)
  {
    $complete_form['#attributes']['class'][] = 'commerce_forteecom-payment-redirect-form';
    $complete_form['#attributes']['class'][] = 'payment-redirect-form';
    $complete_form['#action']                = $element['#return_url'];
    return $element;
  }

  /**
   * @inheritdoc
   *
   * @param array              $form
   * @param FormStateInterface $form_state
   *
   * @return array
   * @throws EntityStorageException
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state)
  {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var PaymentInterface $payment */
    $payment = $this->entity;
    $order   = $payment->getOrder();

    $data['currency_code']     = $payment->getAmount()->getCurrencyCode();
    $data['total']             = $payment->getAmount()->getNumber();
    $data['order_id']          = $payment->getOrderId();
    $data['order_description'] = Drupal::config('system.site')->get('name') . ' - Order: ' . $order->id();
    $data['continueurl']       = $form['#return_url'];
    $data['cancelurl']         = $form['#cancel_url'];

    ForteEcomHelper::check_currency($data['currency_code']);

    /** @var OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $configuration          = $payment_gateway_plugin->getConfiguration();

    /** @var \Drupal\commerce_payment\PaymentMethodStorageInterface $payment_method_storage */
    $payment_method_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment_method');
    $stored_cards           = $payment_method_storage->loadByProperties([
      'payment_gateway' => $payment->getPaymentGatewayId(),
      'uid'             => \Drupal::currentUser()->id()
    ]);

    if ($payment_gateway_plugin->getMode() == 'test') {
        define('FORTEECOM_DEMO_ENABLED',  1);
        $form['#attached']['library'][] = 'commerce_forteecom/checkout_qa';
    } else {
        define('FORTEECOM_DEMO_ENABLED',  0);
        $form['#attached']['library'][] = 'commerce_forteecom/checkout';
    }

    $form['#attached']['drupalSettings']['commerce_forteecom'] = json_encode($data);

    $form['#process'][] = [get_class($this), 'processRedirectForm'];

    $form['#theme']             = 'forteecom-checkout';
    $form['#notify_url']        = $payment_gateway_plugin->getNotifyUrl()->toString();
    $form['#total']             = $data['total'];
    $form['#order_id']          = $data['order_id'];
    $form['#order_description'] = $data['order_description'];
    $form['#currency']          = ForteEcomHelper::get_currency_id($data['currency_code']);
    $form['#stored_cards']      = $stored_cards;
    $form['#payment_gateway']   = $payment->getPaymentGatewayId();
    $form['#card_save_state']   = Drupal::currentUser()->isAuthenticated() ? 'unchecked' : 'hidden';

    $_forteecom_order_data = $order->getData('commerce_forteecom');
    if (!empty($_forteecom_order_data)) {
      $_forteecom_order_id    = $_forteecom_order_data['_forteecom_order_id'];
      $_forteecom_session_id  = $_forteecom_order_data['_forteecom_session_id'];
      $_forteecom_merchant_id = $_forteecom_order_data['_forteecom_merchant_id'];
    } else {

      $init_result = ForteEcomHelper::init_payment($configuration['merchant_id'], $configuration['pre_auth'], $data['total'], $data['order_id'], $data['order_description'], $data['currency_code']);

      $_forteecom_order_id    = $init_result['_forteecom_order_id'];
      $_forteecom_session_id  = $init_result['_forteecom_session_id'];
      $_forteecom_merchant_id = $configuration['merchant_id'];

      ForteEcomHelper::update_order_forteecom_data(
        $order,
        $_forteecom_order_id,
        ForteEcomHelper::STATUS_INIT,
        $_forteecom_session_id,
        $_forteecom_merchant_id
      );

      $order->save();
      $payment->setRemoteId($_forteecom_order_id);
      $payment->setState('inited');
      $payment->setRemoteState(ForteEcomHelper::STATUS_INIT);
      $payment->save();
    }
    $form['#_forteecom_merchant_id'] = $_forteecom_merchant_id;
    $form['#_forteecom_session_id']  = $_forteecom_session_id;
    $form['#_forteecom_order_id']    = $_forteecom_order_id;

    return $form;
  }


}
