<?php

namespace Drupal\commerce_forteecom\PluginForm;

use Drupal\commerce_payment\PluginForm\PaymentMethodAddForm as BasePaymentMethodAddForm;
use Drupal\Core\Form\FormStateInterface;

class PaymentMethodAddForm extends BasePaymentMethodAddForm
{

  /**
   * @inheritDoc
   * All validations are carried by ForteEcom widget
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state)
  {
  }

  /**
   * @inheritdoc
   */
  protected function buildCreditCardForm(array $element, FormStateInterface $form_state)
  {
    $element = parent::buildCreditCardForm($element, $form_state);

    unset($element['security_code'], $element['expiration']['divider']);

    $payment_gateway_plugin = $this->entity->getPaymentGateway()->getPlugin();
    $configuration          = $payment_gateway_plugin->getConfiguration();

    if ($payment_gateway_plugin->getMode() == 'test') {
        define('FORTEECOM_DEMO_ENABLED',  1);
        $form['#attached']['library'][] = 'commerce_forteecom/payment_method_qa';
    } else {
        define('FORTEECOM_DEMO_ENABLED',  0);
        $form['#attached']['library'][] = 'commerce_forteecom/payment_method';
    }

    $element['number'] = [
      '#type'       => 'hidden',
      '#required'   => TRUE,
      '#validated'  => TRUE,
      '#attributes' => [
        'class' => ['forteecom_card_last4']
      ],
    ];

    $element['expiration']['month'] = [
      '#type'       => 'hidden',
      '#required'   => TRUE,
      '#validated'  => TRUE,
      '#attributes' => [
        'class' => ['forteecom_card_expiration_month']
      ],
    ];

    $element['expiration']['year'] = [
      '#type'       => 'hidden',
      '#required'   => TRUE,
      '#validated'  => TRUE,
      '#attributes' => [
        'class' => ['forteecom_card_expiration_year']
      ],
    ];

    $element['forteecom_widget'] = [
      '#type'       => 'html_tag',
      '#tag'        => 'frt-widget-ecom',
      '#required'   => TRUE,
      '#validated'  => TRUE,
      '#attributes' => [
        'inline'      => 'true',
        'mode'        => $payment_gateway_plugin->getMode() == 'test' ? 'demo' : '',
        'merchant-id' => $configuration['merchant_id'],
      ],
    ];

    $element['forteecom_token'] = [
      '#type'       => 'hidden',
      '#required'   => TRUE,
      '#validated'  => TRUE,
      '#attributes' => [
        'class' => ['forteecom_card_token']
      ]
    ];

    $element['type'] = [
      '#type'       => 'hidden',
      '#required'   => TRUE,
      '#validated'  => TRUE,
      '#attributes' => [
        'class' => ['forteecom_card_type']
      ]
    ];


    return $element;
  }

  /**
   * @inheritDoc
   * All validations are carried by ForteEcom widget
   */
  protected function validateCreditCardForm(array &$element, FormStateInterface $form_state)
  {
  }

}
